/*
 * stylus_controller.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licenced for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "cfg.hpp"
#include "stylus_controller.hpp"

class StylusObserver : public AppObserver {
public:
	explicit StylusObserver(StylusController &sc_) : sc(sc_) { }
	virtual void key(Keycode k);
	virtual void keyUp(Keycode k);
	virtual void mouseDown(int x, int y);
	virtual void mouseUp(int x, int y);
	virtual void mouseMove(int x, int y);

private:
	StylusController &sc;
};

StylusController::StylusController(Keycode f1, Keycode f2, Keycode s, App2D &a, int dh)
	: obs(new StylusObserver(*this)), app(a), display_height(dh), fire_key_1(f1),
	  fire_key_2(f2), suicide_key(s),
	  fire1(false), fire2(false), mousedown(false), suicide(false), mx(0), my(0)
{
	app.addObserver(obs.get());
}

StylusController::~StylusController()
{
	app.rmObserver(obs.get());
}

void StylusController::get(MapDirection &dir, bool &centred, bool &fire_held,
						   bool &suicide_held) const
{
	fire_held = fire1 || fire2;
	suicide_held = suicide;
	centred = !mousedown;
	if (!centred) {
		const int dx = mx - CfgInt("mouse_centre_x");
		const int dy = my - (display_height-1-CfgInt("mouse_centre_y"));
		if (abs(dx) > abs(dy)) {
			if (dx < -2) dir = D_WEST;
			else if (dx > 2) dir = D_EAST;
			else centred = true;
		} else {
			if (dy < -2) dir = D_NORTH;
			else if (dy > 2) dir = D_SOUTH;
			else centred = true;
		}
	}
}

void StylusObserver::key(Keycode k)
{
	if (k == sc.fire_key_1) sc.fire1 = true;
	else if (k == sc.fire_key_2) sc.fire2 = true;
	else if (k == sc.suicide_key) sc.suicide = true;
}

void StylusObserver::keyUp(Keycode k)
{
	if (k == sc.fire_key_1) sc.fire1 = false;
	else if (k == sc.fire_key_2) sc.fire2 = false;
	else if (k == sc.suicide_key) sc.suicide = false;
}

void StylusObserver::mouseDown(int x, int y)
{
	sc.mousedown = true;
	sc.mx = x;
	sc.my = y;
}

void StylusObserver::mouseUp(int x, int y)
{
	sc.mousedown = false;
	sc.mx = x;
	sc.my = y;
}

void StylusObserver::mouseMove(int x, int y)
{
	sc.mx = x;
	sc.my = y;
}
